#!/usr/bin/python
# -*- coding: UTF-8 -*-

############################################################################
##                                                                        ##
## XChat Python plugin manager                                            ##
##                                                                        ##
## Copyright (C) 2010  Fabien LOISON (www.flogisoft.com)                  ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################

"""Manage Python module"""

__version__ = "1.0"
__author__ = "Fabien Loison (www.flogisoft.com)"
__copyright__ = "Copyright © 2010 Fabien LOISON"

__module_name__ = "XChat Python plugin manager"
__module_version__ = __version__
__module_description__ = __doc__
__module_author__ = __author__


import os

import xchat


class XChatMenu(object):
    """Provides an XChat menu or submenu"""

    def __init__(self, name, parent=None, sensitive=True, position=-1):
        """The constructor"""
        self._name = name
        self._parent = parent
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._sensitive = sensitive
        self._position = position
        self._children = []
        self._update()

    def set_sensitive(self, sensitive=True):
        """Set if the menu is sensitive or not"""
        self._sensitive = sensitive
        self._update()

    def set_position(self, position):
        """Set the position of the menu"""
        self.remove()
        self._position = position
        self._update()

    def set_name(self, name):
        """Set the name of the menu"""
        self.remove()
        self._name = name
        self._path = name
        self._update()

    def remove(self):
        """Remove the menu"""
        if self._parent != None:
            if self in self._parent._children:
                self._parent._children.remove(self)
        xchat.command("MENU DEL \"%s\"" % self._path)

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #
        cmd += "ADD \"%s\"" % self._path
        #Add menu in XChat
        xchat.command(cmd)
        #Update all children
        for child in self._children:
            child._path = "%s/%s" % (self._path, child._name)
            child._update()


class XChatMenuItem(object):
    """Provides an XChat menu item"""

    def __init__(self, name, parent=None, command="", sensitive=True, position=-1):
        """The constructor"""
        self._parent = parent
        self._name = name
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._command = command
        self._sensitive = sensitive
        self._position = position
        self._update()

    def set_sensitive(self, sensitive=True):
        """Set if the item is sensitive or not"""
        self._sensitive = sensitive
        self._update()

    def set_position(self, position):
        """Set the position of the menu"""
        self.remove()
        self._position = position
        self._update()

    def set_name(self, name):
        """Set the name of the item"""
        self.remove()
        self._name = name
        self._path = "%s/%s" % (self._parent._path, self._name)
        self._update()

    def remove(self):
        """Remove the item"""
        if self._parent != None:
            if self in self._parent._children:
                self._parent._children.remove(self)
        xchat.command("MENU DEL \"%s\"" % self._path)

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #
        cmd += "ADD \"%s\" \"%s\"" % (self._path, self._command)
        xchat.command(cmd)


class XChatMenuCheckbox(XChatMenuItem):
    """Provides an XChat menu item, with a checkbox"""
    
    def __init__(self, name, parent=None, command_check="", command_uncheck="", checked=False, sensitive=True, position=-1):
        """The constructor"""
        self._parent = parent
        self._name = name
        if self._parent == None:
            self._path = self._name
        else:
            self._path = "%s/%s" % (self._parent._path, self._name)
            self._parent._children.append(self)
        self._command_check = command_check
        self._command_uncheck = command_uncheck
        self._checked = checked
        self._sensitive = sensitive
        self._position = position
        self._update()

    def _update(self):
        """Update the menu"""
        cmd = "MENU "
        #Sensitive
        if self._sensitive:
            cmd += "-e1 "
        else:
            cmd += "-e0 "
        #Position
        if self._position >= 0:
            cmd += "-p%i " % self._position
        #Checked
        if self._checked:
            cmd += "-t1 "
        else:
            cmd += "-t0 "
        #
        cmd += "ADD \"%s\" \"%s\" \"%s\"" % (self._path, self._command_check, self._command_uncheck)
        xchat.command(cmd)


def read_conf(conf_file):
    global CONF
    if os.path.isfile(conf_file):
        try:
            file_ = open(conf_file, "r")
            for line in file_:
                if len(line) > 3 and (line[0] == "0" or line[0] == "1") and line[1] == " ":
                    status = line[0]
                    plugin = line[2:].replace("\n", "")
                    if status == "1":
                        CONF[plugin] = True
                    else:
                        CONF[plugin] = False
        except:
            pass
        finally:
            file_.close()


def save_conf(conf_file):
    global CONF
    try:
        file_ = open(conf_file, "w")
        for plugin in CONF:
            file_.write("%s %s\n" % (str(int(CONF[plugin])), plugin))
    except:
        pass
    finally:
        file_.close()


def on_py_cmd(word, word_eol, datas):
    global CONF
    global plugin_conf
    if len(word) == 3:
        action = word[1].lower()
        plugin = word[2]
        if action == "load" or action == "reload":
            CONF[plugin] = True
        else:
            CONF[plugin] = False
    save_conf(plugin_conf)
    return xchat.EAT_NONE


def refresh(word, word_eol, datas):
    global CONF
    global menuitems
    for item in menuitems:
        item.remove()
        del item
    build_menu(list_plugins(plugin_folder), default=False)


def load_plugins(plugin_list):
    global CONF
    for plugin in plugin_list:
        if plugin in CONF:
            if CONF[plugin]:
                xchat.command("py load \"%s\"" % plugin)
        else:
            xchat.command("py load \"%s\"" % plugin)
            CONF[plugin] = True


def list_plugins(path):
    plugins = []
    if os.path.isdir(path):
        files = os.listdir(path)
        for file_ in files:
            if file_[-3:] == ".py":
                plugins.append(os.path.join(path, file_))
    return sorted(plugins)


def build_menu(list, default=True):
    global menuitem
    global CONF
    for plugin in list:
        file_ = plugin.split("/")[-1:][0]
        #Menu name
        if file_[:2].isdigit and file_[2:3] == "_":
            name = file_[3:-3].replace("_", " ")
        else:
            name = file_[:-3].replace("_", " ")
        #Checked ?
        if plugin in CONF:
            if CONF[plugin]:
                checked = True
            else:
                checked = False
        else:
            checked = default
        #Menu
        menuitems.append(XChatMenuCheckbox(
            name,
            menu,
            "py load \"%s\"" % plugin,
            "py unload \"%s\"" % plugin,
            checked
            ))
    menuitems.append(XChatMenuItem("-", menu)) #Separator
    menuitems.append(XChatMenuItem("Refresh", menu, "py_plugins_list_refresh"))


def module_unload(userdata):
    """Called when the module is unloaded"""
    save_conf(plugin_conf)
    print("\0034%s %s [%s] unloaded.\003" %(__module_name__, __module_version__, __copyright__))


print("\0034%s %s [%s] loaded.\003" %(__module_name__, __module_version__, __copyright__))

#Variables
CONF = {}
menuitems = []
plugin_folder = os.path.join(os.environ.get("HOME"), ".xchat2", "plugins")
plugin_conf =  os.path.join(plugin_folder, "plugins.conf")

#Hook
xchat.hook_command("py", on_py_cmd)
xchat.hook_command("py_plugins_list_refresh", refresh)
xchat.hook_unload(module_unload)

##
menu = XChatMenu("Plugins")
read_conf(plugin_conf)
build_menu(list_plugins(plugin_folder))
load_plugins(list_plugins(plugin_folder))


